@@ -1,10 +1,10 @@
-# pylint: disable=too-many-lines
 """Definition of a single parameter
 
 Attributes:
     PARAM_MAPPINGS: The type to Param mappings, used for params to init a
         parameter. Managed by `register_param`
 """
+# pylint: disable=too-many-lines
 import ast
 import json
 from typing import (Optional, List, Any, Callable, Type,
@@ -498,10 +498,11 @@ class ParamInt(Param):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.default = self.default or 0
+        # self.default = self.default or 0
 
     def _value(self) -> int:
-        return int(super()._value())
+        val = super()._value()
+        return None if val is None else int(val)
 
 class ParamFloat(Param):
     """A float parameter whose value is automatically casted into a float"""
@@ -510,10 +511,11 @@ class ParamFloat(Param):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.default = self.default or 0.0
+        # self.default = self.default or 0.0
 
     def _value(self) -> float:
-        return float(super()._value())
+        val = super()._value()
+        return None if val is None else float(val)
 
 class ParamStr(Param):
     """A str parameter whose value is automatically casted into a str"""
@@ -522,7 +524,7 @@ class ParamStr(Param):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.default = self.default or ''
+        # self.default = self.default or ''
 
 class ParamBool(Param):
     """A bool parameter whose value is automatically casted into a bool"""
@@ -531,14 +533,15 @@ class ParamBool(Param):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.default = self.default or False
-        try:
-            cast_to(str(self.default), 'bool')
-        except (PyParamValueError, PyParamTypeError):
-            raise PyParamValueError(
-                "Default value of a count argument must be a bool value or "
-                "a value that can be casted to a bool value."
-            ) from None
+        #self.default = self.default or False
+        if self.default is not None:
+            try:
+                self.default = cast_to(str(self.default), 'bool')
+            except (PyParamValueError, PyParamTypeError):
+                raise PyParamValueError(
+                    "Default value of a count argument must be a bool value or "
+                    "a value that can be casted to a bool value."
+                ) from None
 
     def usagestr(self) -> str:
         """Get the string representation of the parameter in the default usage
@@ -588,11 +591,12 @@ class ParamBool(Param):
             self.push(value)
             return True
 
-    def _value(self) -> bool:
+    def _value(self) -> Optional[bool]:
         if not self._stack:
-            return False
+            return self.default
 
-        ret = cast_to(self._stack[-1][0], 'bool')
+        val = self._stack[-1][0]
+        ret = None if val is None else cast_to(val, 'bool')
         self._stack = []
         return ret
 
@@ -631,7 +635,7 @@ class ParamCount(Param):
         super().__init__(*args, **kwargs)
 
         self.default = self.default or 0
-        if self.default != 0:
+        if self.default is not None and self.default != 0:
             raise PyParamValueError(
                 "Default value of a count argument must be 0"
             )
@@ -846,13 +850,13 @@ class ParamChoice(Param):
             raise PyParamValueError("Argument 'choices' must be "
                                     "a list or a tuple.")
 
-        if not self.required and self.default is None:
-            self.default = self._kwargs['choices'][0]
+        # if not self.required and self.default is None:
+        #     self.default = self._kwargs['choices'][0]
 
     def _value(self) -> Any:
         val = super()._value()
 
-        if val not in self._kwargs['choices']:
+        if val is not None and val not in self._kwargs['choices']:
             raise PyParamValueError(
                 f"{val} is not one of {self._kwargs['choices']}"
             )
@@ -915,16 +919,28 @@ class ParamNamespace(Param):
         """Namespace parameters do not have a default value"""
         return self.desc[:]
 
-    @property
-    def decendents(self) -> List[Type['Param']]:
-        """Get all decendents of this namespace parameter"""
+    def decendents(self, show_only=False) -> List[Type['Param']]:
+        """Get all decendents of this namespace parameter
+
+        Args:
+            show_only: Load params with show=True only?
+
+        Returns:
+            The decendents of this namespace parameter
+        """
         ret: List[Type['Param']] = []
         ret_append: Callable = ret.append
         for param in self._stack.values():
+            # don't skip entire ns parameter
+            if isinstance(param, ParamNamespace):
+                if (not show_only or param.show) and param not in ret:
+                    ret.append(param)
+                ret.extend(param.decendents(show_only))
+                continue
+            if show_only and not param.show:
+                continue
             if param not in ret:
                 ret_append(param)
-            if isinstance(param, ParamNamespace):
-                ret.extend(param.decendents)
         return ret
 
     def consume(self, value: str) -> bool:
@@ -951,6 +967,8 @@ class ParamNamespace(Param):
             if not isinstance(self._stack[part], ParamNamespace):
                 return self._stack[part]
             return self._stack[part].get_param(name, depth + 1)
+        if name in self.names:
+            return self
         return None
 
     # pylint: disable=arguments-differ
