@@ -1,4 +1,5 @@
 """Definition of Params"""
+# pylint: disable=too-many-lines
 import sys
 from typing import Optional, Type, Union, List, Callable, Any, Tuple, Dict
 from pathlib import Path
@@ -6,12 +7,18 @@ from diot import OrderedDiot, Diot
 import rich
 from simpleconf import Config, LOADERS
 from .utils import (
-    always_list, Namespace, logger, parse_type,
-    type_from_value, parse_potential_argument
+    always_list,
+    Namespace,
+    logger,
+    parse_type,
+    type_from_value,
+    parse_potential_argument
 )
-from .defaults import POSITIONAL
+from .defaults import (POSITIONAL,
+                       PARAMS as PARAMS_DEFAULT,
+                       PARAM as PARAM_DEFAULT)
 from .param import PARAM_MAPPINGS
-from .help import HelpAssembler
+from .help import HelpAssembler, ProgHighlighter
 from .completer import Completer
 from .exceptions import PyParamNameError
 
@@ -31,6 +38,7 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             subcommands
         help_on_void: Whether to show help when no arguments provided
         help_callback: A function to modify the help page
+        help_modifier: A callback function to modify the help param/command
         prefix: The prefix for the arguments
             (see attribute `Params.prefix`)
         arbitrary: Whether to parse the command line arbitrarily
@@ -61,21 +69,32 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
     """
     def __init__(self, # pylint: disable=too-many-arguments
                  names: Optional[Union[str, List[str]]] = None,
-                 desc: Optional[Union[List[str], str]] = 'No description',
+                 desc: Optional[Union[List[str], str]] = None,
                  prog: Optional[str] = None,
-                 help_keys: Union[str, List[str]] = 'h,help',
-                 help_cmds: Union[str, List[str]] = 'help',
-                 help_on_void: bool = True,
+                 help_keys: Union[str, List[str]] = None,
+                 help_cmds: Union[str, List[str]] = None,
+                 help_on_void: bool = None,
                  help_callback: Optional[Callable] = None,
+                 help_modifier: Optional[Callable] = None,
                  prefix: str = 'auto',
                  arbitrary: bool = False,
                  theme: str = 'default',
                  usage: Optional[Union[str, List[str]]] = None) -> None:
-        self.desc: List[str] = always_list(desc, strip=False, split=False)
-        self.prog: str = sys.argv[0] if prog is None else prog
-        self.help_keys: List[str] = always_list(help_keys)
-        self.help_cmds: List[str] = always_list(help_cmds)
-        self.help_on_void: bool = help_on_void
+        self.names: List[str] = always_list(names) if names else []
+        self.desc: List[str] = (PARAMS_DEFAULT.desc if desc is None
+                                else always_list(desc,
+                                                 strip=False,
+                                                 split=False))
+        self._prog: str = sys.argv[0] if prog is None else prog
+        self.help_keys: List[str] = (PARAMS_DEFAULT.help_keys
+                                     if help_keys is None
+                                     else always_list(help_keys))
+        self.help_cmds: List[str] = (PARAMS_DEFAULT.help_cmds
+                                     if help_cmds is None
+                                     else always_list(help_cmds))
+        self.help_on_void: bool = (PARAMS_DEFAULT.help_on_void
+                                   if help_on_void is None
+                                   else help_on_void)
         self.usage: Optional[List[str]] = (
             None if usage is None
             else always_list(usage, strip=True, split='\n')
@@ -83,7 +102,6 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         self.prefix: str = prefix
         self.arbitrary: bool = arbitrary
         self.theme: Union[str, rich.theme.Theme] = theme
-        self.names: List[str] = always_list(names) if names else []
 
         self.params: OrderedDiot = OrderedDiot()
         self.commands: OrderedDiot = OrderedDiot()
@@ -91,11 +109,31 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         self.param_groups: OrderedDiot = OrderedDiot()
         self.command_groups: OrderedDiot = OrderedDiot()
 
+        self.help_modifier = help_modifier
+
         self.assembler: HelpAssembler = HelpAssembler(
             self.prog, theme, help_callback
         )
         super().__init__()
 
+    @property
+    def prog(self) -> str:
+        """Get the program name"""
+        return self._prog
+
+    @prog.setter
+    def prog(self, value: str):
+        """Set the program name and update the help assembler
+
+        Args:
+            value: The new program name
+        """
+        self._prog = value
+        self.assembler.console.meta.prog = value
+        self.assembler.console.meta.highlighters.prog = ProgHighlighter(
+            value
+        )
+
     def name(self, which: str = 'short') -> str:
         """Get the shortest/longest name of the parameter
 
@@ -176,13 +214,13 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                   # pylint: disable=redefined-builtin
                   type: Optional[str] = None,
                   desc: Union[str, List[str]] = None,
-                  show: bool = True,
-                  required: bool = False,
+                  show: bool = None,
+                  required: bool = None,
                   callback: Optional[Callable] = None,
                   group: Optional[str] = None,
                   force: bool = False,
-                  type_fronzen: bool = True,
-                  argname_shorten: bool = True,
+                  type_frozen: bool = None,
+                  argname_shorten: bool = None,
                   complete_callback: Optional[Callable] = None,
                   **kwargs) -> Type["Param"]:
         """Add an argument
@@ -242,21 +280,27 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             # Type: Optional[str], Optional[str]
             maintype, subtype = parse_type(type.__name__
                                            if callable(type)
+                                           else PARAM_DEFAULT.type
+                                           if type is None
                                            else type)
 
             param: Type['Param'] = PARAM_MAPPINGS[maintype](
                 names=names,
-                default=default,
-                desc=None if desc is None else always_list(desc,
-                                                           strip=False,
-                                                           split=False),
+                default=PARAM_DEFAULT.default if default is None else default,
+                desc=PARAM_DEFAULT.desc if desc is None else always_list(
+                    desc, strip=False, split=False
+                ),
                 prefix=self.prefix,
-                show=show,
-                required=required,
+                show=PARAM_DEFAULT.show if show is None else show,
+                required=(PARAM_DEFAULT.required if required is None
+                          else required),
                 subtype=subtype,
-                type_fronzen=type_fronzen,
+                type_frozen=(PARAM_DEFAULT.type_frozen if type_frozen is None
+                             else type_frozen),
                 callback=callback,
-                argname_shorten=argname_shorten,
+                argname_shorten=(PARAM_DEFAULT.argname_shorten
+                                 if argname_shorten is None
+                                 else argname_shorten),
                 complete_callback=complete_callback,
                 **kwargs
             )
@@ -300,6 +344,7 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                     help_cmds: Union[str, List[str]] = '__inherit__',
                     help_on_void: Union[str, bool] = '__inherit__',
                     help_callback: Optional[Callable] = None,
+                    help_modifier: Optional[Callable] = None,
                     prefix: str = '__inherit__',
                     arbitrary: Union[str, bool] = '__inherit__',
                     theme: Union[str, rich.theme.Theme] = '__inherit__',
@@ -348,6 +393,7 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                 help_on_void=(self.help_on_void if help_on_void == '__inherit__'
                               else help_on_void),
                 help_callback=help_callback,
+                help_modifier=help_modifier,
                 prefix=(self.prefix if prefix == '__inherit__'
                         else prefix),
                 arbitrary=(self.arbitrary
@@ -415,7 +461,6 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         for param_name, param in self.params.items():
             if param.is_help or param_name in namespace:
                 continue
-
             try:
                 value: Any = param.apply_callback(ns_no_callback)
             except (TypeError, ValueError) as pte:
@@ -424,18 +469,14 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             else:
                 for name in param.names:
                     setattr(namespace, name, value)
-
         return namespace
 
     def parse(self,
-              args: Optional[List[str]] = None,
-              help_modifier: Optional[Callable] = None) -> Namespace:
+              args: Optional[List[str]] = None) -> Namespace:
         """Parse the arguments from the command line
 
         Args:
             args: The arguments to parse
-            help_modifier: A callback to modify the help parameter and command.
-                It takes 2 elements, help parameter and command if any.
 
         Return:
             Namespace: The namespace of parsed arguments
@@ -461,8 +502,8 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                 desc="Command name to print help for"
             )
 
-        if callable(help_modifier):
-            help_modifier(self.get_param(self.help_keys[0]), help_cmd)
+        if callable(self.help_modifier):
+            self.help_modifier(self.get_param(self.help_keys[0]), help_cmd)
 
         if args is None:
             # enable completion only when we are trying to parse sys.argv
@@ -546,6 +587,7 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                     # 1. hits a command
                     # 2. hits the start of positional arguments
                     # Type: Optional[Type['Param']], Optional[str]
+                    prev_param.close()
                     prev_param, matched = self._match_command_or_positional(
                         prev_param, param_value, args[(i+1):], namespace
                     )
@@ -733,6 +775,99 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             param.push(param_value)
         return param, param_name, param_type, param_value
 
+    def _all_params(self, show_only=False) -> List[Type['Param']]:
+        """All parameters under this command
+
+        self.params don't have all of them, since namespaced ones are under
+        namespace paramters
+        """
+        ret: List[Type['Param']] = []
+        ret_append: Callable = ret.append
+        for param in self.params.values():
+            if param.type == 'ns':
+                if (not show_only or param.show) and param not in ret:
+                    ret.append(param)
+                ret.extend(param.decendents(show_only))
+
+            elif show_only and not param.show:
+                continue
+            elif param not in ret:
+                ret_append(param)
+        return ret
+
+    def _to_dict_params(self) -> Diot:
+        """Load all parameters into a dictionary"""
+        ret = Diot()
+        param_groups = {}
+        for group, param_list in self.param_groups.items():
+            for param in param_list:
+                param_groups[param.names[0]] = group
+
+        for param in self._all_params():
+            param_name: str = ("POSITIONAL" if param.names[0] == POSITIONAL
+                               else param.names[0])
+            ret[param_name] = Diot()
+            param_dict: Diot = ret[param_name]
+            if len(param.names) > 1:
+                param_dict.aliases = ["POSITIONAL" if name == POSITIONAL
+                                      else name for name in param.names[1:]]
+            if param.default != PARAM_DEFAULT.default:
+                param_dict.default = param.default
+            if param.typestr() != PARAM_DEFAULT.type:
+                param_dict.type = param.typestr()
+            if param.desc != PARAM_DEFAULT.desc:
+                param_dict.desc = param.desc
+            if param.show != PARAM_DEFAULT.show:
+                param_dict.show = param.show
+            if param.required != PARAM_DEFAULT.required:
+                param_dict.required = param.required
+            if param.type_frozen != PARAM_DEFAULT.type_frozen:
+                param_dict.type_frozen = param.type_frozen
+            if param.argname_shorten != PARAM_DEFAULT.argname_shorten:
+                param_dict.argname_shorten = param.argname_shorten
+            param_dict.group = group = param_groups[param.names[0]]
+            param_dict |= param._kwargs
+
+        return ret
+
+    def _to_dict_commands(self) -> Diot:
+        """Load all commands into a dictionary"""
+        ret = Diot()
+        command_groups = {}
+        for group, command_list in self.command_groups.items():
+            for command in command_list:
+                for name in command.names:
+                    command_groups[name] = group
+
+        for command_name, command in self.commands.items():
+            if any(name in ret for name in command.names):
+                continue
+            ret[command_name] = Diot()
+            cmd_dict: Diot = ret[command_name]
+            if len(command.names) > 1:
+                cmd_dict.aliases = [name for name in command.names
+                                    if name != command_name]
+            if command.desc != PARAMS_DEFAULT.desc:
+                cmd_dict.desc = command.desc
+            if command.help_keys != PARAMS_DEFAULT.help_keys:
+                cmd_dict.help_keys = command.help_keys
+            if command.help_cmds != PARAMS_DEFAULT.help_cmds:
+                cmd_dict.help_cmds = command.help_cmds
+            if command.help_on_void != PARAMS_DEFAULT.help_on_void:
+                cmd_dict.help_on_void = command.help_on_void
+            if command.prefix != PARAMS_DEFAULT.prefix:
+                cmd_dict.prefix = command.prefix
+            if command.arbitrary != PARAMS_DEFAULT.arbitrary:
+                cmd_dict.arbitrary = command.arbitrary
+            if command.theme != PARAMS_DEFAULT.theme:
+                cmd_dict.theme = command.theme
+            if command.usage != PARAMS_DEFAULT.usage:
+                cmd_dict.usage = command.usage
+            cmd_dict.group = command_groups[command_name]
+            cmd_dict |= command.to_dict()
+
+        return ret
+
     def to_dict(self) -> Diot:
         """Save the parameters/commands to file.
 
@@ -743,55 +878,8 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             The complied Diot of parameters and commands
         """
         # load all parameters and commands
-        loaded: Diot = Diot(params={}, commands={})
-        param_groups = {}
-        for group, param_list in self.param_groups.items():
-            for param in param_list:
-                param_groups[param.names[0]] = group
-
-        for param in self._all_params():
-            param_name: str = ("POSITIONAL" if param.names[0] == POSITIONAL
-                               else param.names[0])
-            loaded.params[param_name] = Diot()
-            param_dict: Diot = loaded.params[param_name]
-            if len(param.names) > 1:
-                param_dict.aliases = ["POSITIONAL" if name == POSITIONAL
-                                      else name for name in param.names[1:]]
-            param_dict.default = param.default
-            param_dict.type = param.typestr()
-            param_dict.desc = param.desc
-            param_dict.show = param.show
-            param_dict.required = param.required
-            param_dict.type_frozen = param.type_frozen
-            param_dict.argname_shorten = param.argname_shorten
-            param_dict.group = group = param_groups[param.names[0]]
-
-        command_groups = {}
-        for group, command_list in self.command_groups.items():
-            for command in command_list:
-                for name in command.names:
-                    command_groups[name] = group
-
-        for command_name, command in self.commands.items():
-            if any(name in loaded.commands for name in command.names):
-                continue
-            loaded.commands[command_name] = Diot()
-            cmd_dict: Diot = loaded.commands[command_name]
-            if len(command.names) > 1:
-                cmd_dict.aliases = [name for name in command.names
-                                    if name != command_name]
-            cmd_dict.desc = command.desc
-            cmd_dict.help_keys = command.help_keys
-            cmd_dict.help_cmds = command.help_cmds
-            cmd_dict.help_on_void = command.help_on_void
-            cmd_dict.prefix = command.prefix
-            cmd_dict.arbitrary = command.arbitrary
-            cmd_dict.theme = command.theme
-            cmd_dict.usage = command.usage
-            cmd_dict.group = command_groups[command_name]
-            cmd_dict |= command.to_dict()
-
-        return loaded
+        return Diot(params=self._to_dict_params(),
+                    commands=self._to_dict_commands())
 
     def to_file(self,
                 path: Union[str, Path],
@@ -997,3 +1085,67 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         )
         if filepath:
             self.from_file(filepath, show=show, force=force)
+
+    def __repr__(self):
+        return "<Params(%s) @ %s>" % (','.join(self.names), id(self))
+
+    def copy(self, deep=False) -> "Params":
+        """Copy a Params object
+
+        Args:
+            deep: Whether to copy the parameters and commands deeply
+
+        Returns:
+            The copied params object
+        """
+        copied: "Params" = Params(
+            names=self.names[:],
+            desc=self.desc[:],
+            prog=self.prog,
+            help_keys=self.help_keys[:],
+            help_cmds=self.help_cmds[:],
+            help_on_void=self.help_on_void,
+            help_callback=None,
+            help_modifier=self.help_modifier,
+            prefix=self.prefix,
+            arbitrary=self.arbitrary,
+            theme=self.theme,
+            usage=self.usage and self.usage[:]
+        )
+        copied.assembler = self.assembler
+
+        if not deep:
+            copied.params = self.params.copy()
+            copied.commands = self.commands.copy()
+
+            copied.param_groups = self.param_groups.copy()
+            copied.command_groups = self.command_groups.copy()
+        else:
+            copied.params = OrderedDiot()
+            copied.commands = OrderedDiot()
+
+            copied.param_groups = OrderedDiot()
+            copied.command_groups = OrderedDiot()
+
+            for param in self.params.values():
+                if any(name in copied.params for name in param.names):
+                    continue
+                param_copy = param.copy()
+                for name in param.names:
+                    copied.params[name] = param_copy
+
+            for command in self.commands.values():
+                if any(name in copied.commands for name in command.names):
+                    continue
+                command_copy = command.copy(deep=deep)
+                for name in command.names:
+                    copied.commands[name] = command_copy
+
+            for group, param_list in self.param_groups.items():
+                self.param_groups[group] = [copied.get_param(param.names[0])
+                                            for param in param_list]
+            for group, cmd_list in self.command_groups.items():
+                self.command_groups[group] = [copied.commands[cmd.names[0]]
+                                              for cmd in cmd_list]
+
+        return copied
